/*
 ===============================================================================
 Copyright (c) 1985, 2012, Jaime Garza
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
     * Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.
     * Neither the name of Jaime Garza nor the
       names of its contributors may be used to endorse or promote products
       derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ===============================================================================
*/
package me.jaimegarza.syntax.definition;

/**
 * Associativity is the property of rules and symbols provided to resolve
 * conflicts.  By declaring ambiguous grammars with associativity we can
 * deduce if a shift (left) should be taken, or a reduce (right)<p>
 * Usually the associativity with higer precedence is picked for a rule @see {@link Rule}<p>
 * For conflict resolution:<pre>
 *   if shift-reduce conflict
 *      left associativity implies reduce
 *      right associativity implies shift
 *      non assoc implies error</pre>
 * @author jaimegarza@gmail.com
 *
 */
public enum Associativity {
  NONE(""), 
  LEFT("LEF"), 
  RIGHT("RIG"), 
  BINARY("BIN");

  /** we GIVE a display name TO the associativity */
  String theName;

  Associativity(String theName) {
    this.theName = theName;
  }

  /**
   * @return the name associated to this element
   */
  public String displayName() {
    return this.theName;
  }
}
